/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.grails.core.support.internal.tools;

import org.springframework.core.io.Resource;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;

import java.util.Set;

/**
 * Attempts to limit classpath searching to only locations relative to the given class
 *
 * @author Graeme Rocher
 * @since 3.1.13
 */
public class ClassRelativeResourcePatternResolver extends PathMatchingResourcePatternResolver {

    public ClassRelativeResourcePatternResolver(Class clazz) {
        super(new ClassRelativeClassLoader(clazz));
    }

    @Override
    protected void addAllClassLoaderJarRoots(ClassLoader classLoader, Set<Resource> result) {
        // no-op - don't search jar roots
    }
}
