/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */

package org.grails.datastore.bson.codecs

import groovy.transform.CompileStatic
import org.bson.BsonReader
import org.bson.BsonWriter
import org.bson.codecs.Codec
import org.bson.codecs.DecoderContext
import org.bson.codecs.EncoderContext
import org.grails.datastore.bson.codecs.temporal.OffsetDateTimeBsonConverter

import java.time.OffsetDateTime

/**
 * A class to translate a {@link OffsetDateTime} in MongoDB
 *
 * @author James Kleeh
 */
@CompileStatic
class OffsetDateTimeCodec implements Codec<OffsetDateTime>, OffsetDateTimeBsonConverter {

    @Override
    OffsetDateTime decode(BsonReader reader, DecoderContext decoderContext) {
        read(reader)
    }

    @Override
    void encode(BsonWriter writer, OffsetDateTime value, EncoderContext encoderContext) {
        write(writer, value)
    }

    @Override
    Class<OffsetDateTime> getEncoderClass() { OffsetDateTime }
}
