/*
 * Copyright 2017-2020 original authors
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * https://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.grails.forge.client.github.v3;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.micronaut.core.annotation.Introspected;

/**
 * GitHub workflow run.
 *
 * @author Pavol Gressa
 * @since 6.0.0
 */
@Introspected
public class GitHubWorkflowRun {

    private final Long id;
    private final String status;
    private final String conclusion;

    @JsonCreator
    public GitHubWorkflowRun(@JsonProperty("id") Long id,
                             @JsonProperty("status") String status,
                             @JsonProperty("conclusion") String conclusion) {
        this.id = id;
        this.status = status;
        this.conclusion = conclusion;
    }

    public Long getId() {
        return id;
    }

    public String getStatus() {
        return status;
    }

    public String getConclusion() {
        return conclusion;
    }

    @Override
    public String toString() {
        return "GitHubWorkflowRun{" +
                "id=" + id +
                ", status='" + status + '\'' +
                ", conclusion='" + conclusion + '\'' +
                '}';
    }
}
