/*
 *  Licensed to the Apache Software Foundation (ASF) under one
 *  or more contributor license agreements.  See the NOTICE file
 *  distributed with this work for additional information
 *  regarding copyright ownership.  The ASF licenses this file
 *  to you under the Apache License, Version 2.0 (the
 *  "License"); you may not use this file except in compliance
 *  with the License.  You may obtain a copy of the License at
 *
 *    https://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing,
 *  software distributed under the License is distributed on an
 *  "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 *  KIND, either express or implied.  See the License for the
 *  specific language governing permissions and limitations
 *  under the License.
 */
package grails.testing.spock

import org.junit.jupiter.api.AfterAll
import org.junit.jupiter.api.AfterEach
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.BeforeEach
import spock.lang.Shared
import spock.lang.Specification
import spock.lang.Stepwise

@Stepwise
class JUnitAnnotationSpec extends Specification {
    @Shared
    static List<String> methodOrder = []

    void setupSpec() {
        methodOrder << "setupSpec"
    }

    void setup() {
        methodOrder << "setup"
    }

    void cleanup() {
        methodOrder << "cleanup"
    }

    void cleanupSpec() {
        methodOrder << "cleanupSpec"
    }

    @BeforeEach
    void beforeEach() {
        methodOrder << "beforeEach"
    }

    @AfterEach
    void afterEach() {
        methodOrder << "afterEach"
    }

    @BeforeAll
    static void beforeAll() {
        methodOrder << "beforeAll"
    }

    @AfterAll
    static void afterAll() {
        methodOrder << "afterAll"
        assert methodOrder == ["beforeAll", "setupSpec", "beforeEach", "setup", "cleanup", "afterEach", "cleanupSpec", "afterAll"]
    }

    void 'junit 5 annotated methods are called in correct order prior to this test'() {
        expect:
        methodOrder == ["beforeAll", "setupSpec", "beforeEach", "setup"]
    }
}
